/*
* AW2UnitDamageEditor.java
*/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

/**
* @author CodeReaper, converted to AW2 by Xenesis
*/
public final class AW2UnitDamageEditor extends JFrame
{
    public static void main(String[]args)
    {
        RandomAccessFile gameFile = null;
        
        JFileChooser fileChooser = new JFileChooser();
        
        int open = fileChooser.showOpenDialog( null );
        
        try
        {
            gameFile = new RandomAccessFile( fileChooser.getSelectedFile().toString(), "rw" );
        }
        catch( NullPointerException ex )
        {
            System.exit(0);
        }
        catch( FileNotFoundException ex )
        {
            System.exit(0);
        }
        
        AW2UnitDamageEditor editor = new AW2UnitDamageEditor( gameFile );
    }
    
    private static final int
            START_INDEX = 0x5D5ABC,
		INFANTRY_INDEX = 1,
		MECH_INDEX = 2,
		MDTANK_INDEX = 3,
		DUMMY1_INDEX = 4,
		TANK_INDEX = 5,
		RECON_INDEX = 6,
		APC_INDEX = 7,
		NEOTANK_INDEX = 8,
		DUMMY2_INDEX = 9,
		ARTILLERY_INDEX = 0xA,
		ROCKET_INDEX = 0xB,
		DUMMY3_INDEX = 0xC,
		DUMMY4_INDEX = 0xD,
		AA_INDEX = 0xE,
		MISSILE_INDEX = 0xF,
		FIGHTER_INDEX = 0x10,
		BOMBER_INDEX = 0x11,
		DUMMY5_INDEX = 0x12,
		BCOPTER_INDEX = 0x13,
		TCOPTER_INDEX = 0x14,
		BATTLESHIP_INDEX = 0x15,
		CRUISER_INDEX = 0x16,
		LANDER_INDEX = 0x17,
		SUB_INDEX = 0x18,
		DIVEDSUB_INDEX = 0x19,
		PRIMARY_OFFSET = 0x1E,
		SECONDARY_OFFSET = 0x38;
    
    private static final int
            LABEL_WIDTH = 70,
            LABEL_HEIGHT = 20,
            BOX_WIDTH = 40,
            BOX_HEIGHT = 20,
            BUTTON_HEIGHT = 20,
            NUM_ROWS = 48,
            NUM_COLS = 25,
            WINDOW_WIDTH = LABEL_WIDTH + ( BOX_WIDTH * NUM_COLS ) +5,
            WINDOW_HEIGHT = LABEL_HEIGHT + ( BOX_HEIGHT * NUM_ROWS ) + BUTTON_HEIGHT;
    
    private static final String
            TITLE = "AW2 Unit Damage Editor",
            SAVE = "Save",
		INFANTRY = "Infantry",
		MECH = "Mech",
		MDTANK = "Md Tank",
		DUMMY1 = "Dummy 1",
		TANK = "Tank",
		RECON = "Recon",
		APC = "APC",
		NEOTANK = "Neotank",
		DUMMY2 = "Dummy 2",
		ARTILLERY = "Artillery",
		ROCKET = "Rocket",
		DUMMY3 = "Dummy 3",
		DUMMY4 = "Dummy 4",
		AA = "AA",
		MISSILE = "Missile",
		FIGHTER = "Fighter",
		BOMBER = "Bomber",
		DUMMY5 = "Dummy 5",
		BCOPTER = "B Copter",
		TCOPTER = "T Copter",
		BATTLESHIP = "B Ship",
		CRUISER = "Cruiser",
		LANDER = "Lander",
		SUB = "Sub",
		DIVEDSUB = "Sub (Dive)",
		INFANTRYA = "Infantry*",
		MECHA = "Mech*",
		MDTANKA = "Md Tank*",
		DUMMY1A = "Dummy 1*",
		TANKA = "Tank*",
		RECONA = "Recon*",
		APCA = "APC*",
		NEOTANKA = "Neotank*",
		DUMMY2A = "Dummy 2*",
		ARTILLERYA = "Artillery*",
		ROCKETA = "Rocket*",
		DUMMY3A = "Dummy 3*",
		DUMMY4A = "Dummy 4*",
		AAA = "AA*",
		MISSILEA = "Missile*",
		FIGHTERA = "Fighter*",
		BOMBERA = "Bomber*",
		DUMMY5A = "Dummy 5*",
		BCOPTERA = "B Copter*",
		TCOPTERA = "T Copter*",
		BATTLESHIPA = "B Ship*",
		CRUISERA = "Cruiser*",
		LANDERA = "Lander*",
		SUBA = "Sub*",
		DIVEDSUBA = "Sub (Dive)*",
		INFANTRYS = "Inf",
		MECHS = "Mech",
		MDTANKS = "Md Tk",
		DUMMY1S = "Duy1",
		TANKS = "Tank",
		RECONS = "Recon",
		APCS = "APC",
		NEOTANKS = "Neo",
		DUMMY2S = "Duy2",
		ARTILLERYS = "Arty",
		ROCKETS = "Rockt",
		DUMMY3S = "Duy3",
		DUMMY4S = "Duy4",
		AAS = "AA",
		MISSILES = "Miss",
		FIGHTERS = "Fight",
		BOMBERS = "Bomb",
		DUMMY5S = "Duy5",
		BCOPTERS = "BCopt",
		TCOPTERS = "TCopt",
		BATTLESHIPS = "BShip",
		CRUISERS = "Crusr",
		LANDERS = "Landr",
		SUBS = "Sub",
		DIVEDSUBS = "Sub D";
;
    
    private static final int[] WEAPON_OFFSETS = defineWeaponOffsets();
    
    private static int[] defineWeaponOffsets()
    {
        int[] weapon_offsets = new int[NUM_ROWS];
        
        weapon_offsets[0] = INFANTRY_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[1] = INFANTRY_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[2] = MECH_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[3] = MECH_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[4] = MDTANK_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[5] = MDTANK_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[6] = DUMMY1_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[7] = DUMMY1_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[8] = TANK_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[9] = TANK_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[10] = RECON_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[11] = RECON_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[12] = APC_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[13] = APC_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[14] = NEOTANK_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[15] = NEOTANK_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[16] = DUMMY2_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[17] = DUMMY2_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[18] = ARTILLERY_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[19] = ARTILLERY_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[20] = ROCKET_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[21] = ROCKET_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[22] = DUMMY3_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[23] = DUMMY3_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[24] = DUMMY4_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[25] = DUMMY4_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[26] = AA_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[27] = AA_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[28] = MISSILE_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[29] = MISSILE_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[30] = FIGHTER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[31] = FIGHTER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[32] = BOMBER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[33] = BOMBER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[34] = DUMMY5_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[35] = DUMMY5_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[36] = BCOPTER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[37] = BCOPTER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[38] = TCOPTER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[39] = TCOPTER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[40] = BATTLESHIP_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[41] = BATTLESHIP_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[42] = CRUISER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[43] = CRUISER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[44] = LANDER_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[45] = LANDER_INDEX *0x5C + SECONDARY_OFFSET;
        weapon_offsets[46] = SUB_INDEX *0x5C + PRIMARY_OFFSET;
        weapon_offsets[47] = SUB_INDEX *0x5C + SECONDARY_OFFSET;
        
        return weapon_offsets;
    }
    
    private static final int[] UNIT_OFFSETS = defineUnitOffsets();
    
    private static int[] defineUnitOffsets()
    {
        int[] unit_offsets = new int[NUM_COLS];
        
        unit_offsets[0] = 		INFANTRY_INDEX;
        unit_offsets[1] = 		MECH_INDEX;
        unit_offsets[2] = 		MDTANK_INDEX;
        unit_offsets[3] = 		DUMMY1_INDEX;
        unit_offsets[4] = 		TANK_INDEX;
        unit_offsets[5] = 		RECON_INDEX;
        unit_offsets[6] = 		APC_INDEX;
        unit_offsets[7] = 		NEOTANK_INDEX;
        unit_offsets[8] = 		DUMMY2_INDEX;
        unit_offsets[9] = 		ARTILLERY_INDEX;
        unit_offsets[10] = 		ROCKET_INDEX;
        unit_offsets[11] = 		DUMMY3_INDEX;
        unit_offsets[12] = 		DUMMY4_INDEX;
        unit_offsets[13] = 		AA_INDEX;
        unit_offsets[14] = 		MISSILE_INDEX;
        unit_offsets[15] = 		FIGHTER_INDEX;
        unit_offsets[16] = 		BOMBER_INDEX;
        unit_offsets[17] = 		DUMMY5_INDEX;
        unit_offsets[18] = 		BCOPTER_INDEX;
        unit_offsets[19] = 		TCOPTER_INDEX;
        unit_offsets[20] = 		BATTLESHIP_INDEX;
        unit_offsets[21] = 		CRUISER_INDEX;
        unit_offsets[22] = 		LANDER_INDEX;
        unit_offsets[23] = 		SUB_INDEX;
        unit_offsets[24] = 		DIVEDSUB_INDEX;
        
        return unit_offsets;
    }
    
    private static final String[] WEAPON_NAMES = defineWeaponNames();
    
    private static String[] defineWeaponNames()
    {
        String[] weapon_names = new String[NUM_ROWS];
        
        weapon_names[0] = INFANTRY;
        weapon_names[1] = INFANTRYA;
        weapon_names[2] = MECH;
        weapon_names[3] = MECHA;
        weapon_names[4] = MDTANK;
        weapon_names[5] = MDTANKA;
        weapon_names[6] = DUMMY1;
        weapon_names[7] = DUMMY1A;
        weapon_names[8] = TANK;
        weapon_names[9] = TANKA;
        weapon_names[10] = RECON;
        weapon_names[11] = RECONA;
        weapon_names[12] = APC;
        weapon_names[13] = APCA;
        weapon_names[14] = NEOTANK;
        weapon_names[15] = NEOTANKA;
        weapon_names[16] = DUMMY2;
        weapon_names[17] = DUMMY2A;
        weapon_names[18] = ARTILLERY;
        weapon_names[19] = ARTILLERYA;
        weapon_names[20] = ROCKET;
        weapon_names[21] = ROCKETA;
        weapon_names[22] = DUMMY3;
        weapon_names[23] = DUMMY3A;
        weapon_names[24] = DUMMY4;
        weapon_names[25] = DUMMY4A;
        weapon_names[26] = AA;
        weapon_names[27] = AAA;
        weapon_names[28] = MISSILE;
        weapon_names[29] = MISSILEA;
        weapon_names[30] = FIGHTER;
        weapon_names[31] = FIGHTERA;
        weapon_names[32] = BOMBER;
        weapon_names[33] = BOMBERA;
        weapon_names[34] = DUMMY5;
        weapon_names[35] = DUMMY5A;
        weapon_names[36] = BCOPTER;
        weapon_names[37] = BCOPTERA;
        weapon_names[38] = TCOPTER;
        weapon_names[39] = TCOPTERA;
        weapon_names[40] = BATTLESHIP;
        weapon_names[41] = BATTLESHIPA;
        weapon_names[42] = CRUISER;
        weapon_names[43] = CRUISERA;
        weapon_names[44] = LANDER;
        weapon_names[45] = LANDERA;
        weapon_names[46] = SUB;
        weapon_names[47] = SUBA;

        
        return weapon_names;
    }
    
    private static final String[] UNIT_NAMES = defineUnitNames();
    
    private static String[] defineUnitNames()
    {
        String[] unit_names = new String[NUM_COLS];
        
        unit_names[0] = INFANTRYS;
        unit_names[1] = MECHS;
        unit_names[2] = MDTANKS;
        unit_names[3] = DUMMY1S;
        unit_names[4] = TANKS;
        unit_names[5] = RECONS;
        unit_names[6] = APCS;
        unit_names[7] = NEOTANKS;
        unit_names[8] = DUMMY2S;
        unit_names[9] = ARTILLERYS;
        unit_names[10] = ROCKETS;
        unit_names[11] = DUMMY3S;
        unit_names[12] = DUMMY4S;
        unit_names[13] = AAS;
        unit_names[14] = MISSILES;
        unit_names[15] = FIGHTERS;
        unit_names[16] = BOMBERS;
        unit_names[17] = DUMMY5S;
        unit_names[18] = BCOPTERS;
        unit_names[19] = TCOPTERS;
        unit_names[20] = BATTLESHIPS;
        unit_names[21] = CRUISERS;
        unit_names[22] = LANDERS;
        unit_names[23] = SUBS;
	unit_names[24] = DIVEDSUBS;
        
        return unit_names;
    }
    
    private static final Font font = new Font( Font.SANS_SERIF, Font.BOLD, 12 );
    
    private JLabel[] weaponLabels;
    
    private JLabel[] unitLabels;
    
    private JTextField[][] input;
    
    private JButton saveButton;
    
    private RandomAccessFile dataFile;
    
    public AW2UnitDamageEditor( RandomAccessFile file )
    {
        setTitle( TITLE );
        setSize( WINDOW_WIDTH + 6, WINDOW_HEIGHT + 38 );
        setLayout( null );
        
        int i,j;
        
        weaponLabels = new JLabel[NUM_ROWS];
        
        for( i = 0; i < NUM_ROWS; i++ )
        {
            weaponLabels[i] = new JLabel( WEAPON_NAMES[i] );
            weaponLabels[i].setFont( font );
            weaponLabels[i].setBounds( 0, LABEL_HEIGHT + ( i * BOX_HEIGHT ), LABEL_WIDTH, BOX_HEIGHT );
            add( weaponLabels[i] );
        }
        
        unitLabels = new JLabel[NUM_COLS];
        
        for( j = 0; j < NUM_COLS; j++ )
        {
            unitLabels[j] = new JLabel( UNIT_NAMES[j] );
            unitLabels[j].setFont( font );
            unitLabels[j].setBounds( LABEL_WIDTH + ( j * BOX_WIDTH ), 0, BOX_WIDTH, LABEL_HEIGHT );
            add( unitLabels[j] );
        }
        
        dataFile = file;
        
        input = new JTextField[NUM_ROWS][NUM_COLS];
        
        try
        {
            for( i = 0; i < NUM_ROWS; i++ )
            {
                for( j = 0; j < NUM_COLS; j++ )
                {
                    input[i][j] = new JTextField();
                    input[i][j].setFont( font );
                    input[i][j].setBounds( LABEL_WIDTH + ( j * BOX_WIDTH ), LABEL_HEIGHT + ( i * BOX_HEIGHT ), BOX_WIDTH, BOX_HEIGHT );
                    dataFile.seek( START_INDEX + WEAPON_OFFSETS[i] + UNIT_OFFSETS[j] );
                    input[i][j].setText( String.valueOf( dataFile.readByte() ) );
                    add( input[i][j] );
                }
            }
        }
        catch( IOException ex )
        {
            System.err.println( ex.getMessage() );
            System.exit(0);
        }
        
        saveButton = new JButton( SAVE );
        saveButton.setFont( font );
        saveButton.setBounds( 0, LABEL_HEIGHT + ( NUM_ROWS * BOX_HEIGHT ), WINDOW_WIDTH, BUTTON_HEIGHT );
        saveButton.addActionListener( new SaveButtonListener() );
        add( saveButton );
        
        setResizable( false );
        setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        setVisible( true );
    }
    
    public void save()
    {
        int i,j;
        
        try
        {
            for( i = 0; i < NUM_ROWS; i++ )
            {
                for( j = 0; j < NUM_COLS; j++ )
                {
                    dataFile.seek( START_INDEX + WEAPON_OFFSETS[i] + UNIT_OFFSETS[j] );
                    dataFile.writeByte( Integer.parseInt( input[i][j].getText() ) );
                }
            }
        }
        catch( NumberFormatException ex )
        {
            JOptionPane.showMessageDialog( null, "Error!", "Not a valid number", 0 );
        }
        catch( IOException ex )
        {
            System.err.println( ex.getMessage() );
            System.exit(0);
        }
    }
    
    private class SaveButtonListener implements ActionListener
    {
        public void actionPerformed( ActionEvent ae )
        {
            if( ae.getSource() == saveButton )
                save();
        }
    }
}